@@ -158,6 +158,8 @@ typedef struct tagDirectPlay2Data
 
   lpGroupData lpSysGroup; /* System group with _everything_ in it */
 
+  DPID systemPlayerId;
+
   LPDPSESSIONDESC2 lpSessionDesc;
 
   /* I/O Msg queues */
@@ -199,8 +201,8 @@ typedef struct IDirectPlayImpl
   DirectPlay2Data *dp2;
 } IDirectPlayImpl;
 
-HRESULT DP_HandleMessage( IDirectPlayImpl *This, const void *lpMessageBody,
-        DWORD  dwMessageBodySize, const void *lpMessageHeader, WORD wCommandId, WORD wVersion,
+HRESULT DP_HandleMessage( IDirectPlayImpl *This, void *messageBody,
+        DWORD  dwMessageBodySize, void *messageHeader, WORD wCommandId, WORD wVersion,
         void **lplpReply, DWORD *lpdwMsgSize );
 DPSESSIONDESC2 *DP_DuplicateSessionDesc( const DPSESSIONDESC2 *src, BOOL dstAnsi, BOOL srcAnsi );
 HRESULT DP_CreatePlayer( IDirectPlayImpl *This, void *msgHeader, DPID *lpid, DPNAME *lpName,
