@@ -653,44 +653,6 @@ static void HTMLObjectElement_unlink(DispatchEx *dispex)
     unlink_ref(&This->nsobject);
 }
 
-static void HTMLObjectElement_destructor(DispatchEx *dispex)
-{
-    HTMLObjectElement *This = impl_from_DispatchEx(dispex);
-
-    if(This->plugin_container.plugin_host)
-        detach_plugin_host(This->plugin_container.plugin_host);
-
-    HTMLElement_destructor(&This->plugin_container.element.node.event_target.dispex);
-}
-
-static HRESULT HTMLObjectElement_get_dispid(DispatchEx *dispex, const WCHAR *name, DWORD grfdex, DISPID *dispid)
-{
-    HTMLObjectElement *This = impl_from_DispatchEx(dispex);
-
-    TRACE("(%p)->(%s %lx %p)\n", This, debugstr_w(name), grfdex, dispid);
-
-    return get_plugin_dispid(&This->plugin_container, name, dispid);
-}
-
-static HRESULT HTMLObjectElement_dispex_get_name(DispatchEx *dispex, DISPID id, BSTR *name)
-{
-    HTMLObjectElement *This = impl_from_DispatchEx(dispex);
-
-    FIXME("(%p)->(%lx %p)\n", This, id, name);
-
-    return E_NOTIMPL;
-}
-
-static HRESULT HTMLObjectElement_invoke(DispatchEx *dispex, DISPID id, LCID lcid, WORD flags, DISPPARAMS *params,
-        VARIANT *res, EXCEPINFO *ei, IServiceProvider *caller)
-{
-    HTMLObjectElement *This = impl_from_DispatchEx(dispex);
-
-    TRACE("(%p)->(%ld)\n", This, id);
-
-    return invoke_plugin_prop(&This->plugin_container, id, lcid, flags, params, res, ei);
-}
-
 static const NodeImplVtbl HTMLObjectElementImplVtbl = {
     .clsid                 = &CLSID_HTMLObjectElement,
     .cpc_entries           = HTMLElement_cpc,
@@ -703,12 +665,12 @@ static const event_target_vtbl_t HTMLObjectElement_event_target_vtbl = {
     {
         HTMLELEMENT_DISPEX_VTBL_ENTRIES,
         .query_interface= HTMLObjectElement_query_interface,
-        .destructor     = HTMLObjectElement_destructor,
+        .destructor     = HTMLPluginContainer_destructor,
         .traverse       = HTMLObjectElement_traverse,
         .unlink         = HTMLObjectElement_unlink,
-        .get_dispid     = HTMLObjectElement_get_dispid,
-        .get_name       = HTMLObjectElement_dispex_get_name,
-        .invoke         = HTMLObjectElement_invoke
+        .get_dispid     = HTMLPluginContainer_get_dispid,
+        .get_prop_desc  = HTMLPluginContainer_get_prop_desc,
+        .invoke         = HTMLPluginContainer_invoke
     },
     HTMLELEMENT_EVENT_TARGET_VTBL_ENTRIES,
     .handle_event       = HTMLElement_handle_event
@@ -720,12 +682,13 @@ static const tid_t HTMLObjectElement_iface_tids[] = {
     HTMLELEMENT_TIDS,
     0
 };
-static dispex_static_data_t HTMLObjectElement_dispex = {
-    "HTMLObjectElement",
-    &HTMLObjectElement_event_target_vtbl.dispex_vtbl,
-    DispHTMLObjectElement_tid,
-    HTMLObjectElement_iface_tids,
-    HTMLElement_init_dispex_info
+dispex_static_data_t HTMLObjectElement_dispex = {
+    .id           = PROT_HTMLObjectElement,
+    .prototype_id = PROT_HTMLElement,
+    .vtbl         = &HTMLObjectElement_event_target_vtbl.dispex_vtbl,
+    .disp_tid     = DispHTMLObjectElement_tid,
+    .iface_tids   = HTMLObjectElement_iface_tids,
+    .init_info    = HTMLElement_init_dispex_info,
 };
 
 HRESULT HTMLObjectElement_Create(HTMLDocumentNode *doc, nsIDOMElement *nselem, HTMLElement **elem)
@@ -925,12 +888,13 @@ static const tid_t HTMLEmbedElement_iface_tids[] = {
     IHTMLEmbedElement_tid,
     0
 };
-static dispex_static_data_t HTMLEmbedElement_dispex = {
-    "HTMLEmbedElement",
-    &HTMLEmbedElement_event_target_vtbl.dispex_vtbl,
-    DispHTMLEmbed_tid,
-    HTMLEmbedElement_iface_tids,
-    HTMLElement_init_dispex_info
+dispex_static_data_t HTMLEmbedElement_dispex = {
+    .id           = PROT_HTMLEmbedElement,
+    .prototype_id = PROT_HTMLElement,
+    .vtbl         = &HTMLEmbedElement_event_target_vtbl.dispex_vtbl,
+    .disp_tid     = DispHTMLEmbed_tid,
+    .iface_tids   = HTMLEmbedElement_iface_tids,
+    .init_info    = HTMLElement_init_dispex_info,
 };
 
 HRESULT HTMLEmbedElement_Create(HTMLDocumentNode *doc, nsIDOMElement *nselem, HTMLElement **elem)
