@@ -77,7 +77,7 @@ void wineXmlCallbackLog(char const* caller, xmlErrorLevel lvl, char const* msg,
     wine_dbg_log(dbcl, &__wine_dbch_msxml, caller, "%s", buff);
 }
 
-void wineXmlCallbackError(char const* caller, xmlErrorPtr err)
+void wineXmlCallbackError(char const* caller, const xmlError* err)
 {
     enum __wine_debug_class dbcl;
 
@@ -167,7 +167,7 @@ static int to_utf8(int cp, unsigned char *out, int *outlen, const unsigned char
     WCHAR *tmp;
     int len = 0;
 
-    if (!in || !inlen) goto done;
+    if (!in || !inlen || !*inlen) goto done;
 
     len = MultiByteToWideChar(cp, 0, (const char *)in, *inlen, NULL, 0);
     tmp = malloc(len * sizeof(WCHAR));
@@ -187,7 +187,7 @@ static int from_utf8(int cp, unsigned char *out, int *outlen, const unsigned cha
     WCHAR *tmp;
     int len = 0;
 
-    if (!in || !inlen) goto done;
+    if (!in || !inlen || !*inlen) goto done;
 
     len = MultiByteToWideChar(CP_UTF8, 0, (const char *)in, *inlen, NULL, 0);
     tmp = malloc(len * sizeof(WCHAR));
@@ -365,10 +365,8 @@ BOOL WINAPI DllMain(HINSTANCE hInstDLL, DWORD fdwReason, LPVOID reserved)
     case DLL_PROCESS_ATTACH:
         xmlInitParser();
 
-        /* Set the default indent character to a single tab,
-           for this thread and as default for new threads */
+        /* Set the default indent character to a single tab */
         xmlTreeIndentString = "\t";
-        xmlThrDefTreeIndentString("\t");
 
          /* Register callbacks for loading XML files */
         if(xmlRegisterInputCallbacks(wineXmlMatchCallback, wineXmlOpenCallback,
