@@ -2353,19 +2353,11 @@ BOOL WINAPI NtUserGetCaretPos( POINT *pt )
     return ret;
 }
 
-BOOL set_ime_composition_window_pos( HWND hwnd, const POINT *point )
+BOOL set_ime_composition_rect( HWND hwnd, RECT rect )
 {
-    HWND root_hwnd;
-    POINT pt;
-
-    if (!NtUserIsWindow( hwnd ))
-        return FALSE;
-
-    root_hwnd = NtUserGetAncestor( hwnd, GA_ROOT );
-    pt = *point;
-    NtUserMapWindowPoints( hwnd, root_hwnd, &pt, 1, 0 /* per-monitor DPI */ );
-
-    return user_driver->pSetIMECompositionWindowPos( root_hwnd, &pt );
+    if (!NtUserIsWindow( hwnd )) return FALSE;
+    NtUserMapWindowPoints( hwnd, 0, (POINT *)&rect, 2, 0 /* per-monitor DPI */ );
+    return user_driver->pSetIMECompositionRect( NtUserGetAncestor( hwnd, GA_ROOT ), rect );
 }
 
 /*******************************************************************
@@ -2376,7 +2368,6 @@ BOOL set_caret_pos( int x, int y )
     int old_state = 0;
     int hidden = 0;
     HWND hwnd = 0;
-    POINT pt;
     BOOL ret;
     RECT r;
 
@@ -2406,10 +2397,8 @@ BOOL set_caret_pos( int x, int y )
         r.bottom += y - r.top;
         r.left = x;
         r.top = y;
-        pt.x = x;
-        pt.y = y;
         display_caret( hwnd, &r );
-        set_ime_composition_window_pos( hwnd, &pt );
+        set_ime_composition_rect( hwnd, r );
         NtUserSetSystemTimer( hwnd, SYSTEM_TIMER_CARET, caret.timeout );
     }
     return ret;
@@ -2421,7 +2410,6 @@ BOOL set_caret_pos( int x, int y )
 BOOL WINAPI NtUserShowCaret( HWND hwnd )
 {
     int hidden = 0;
-    POINT pt;
     BOOL ret;
     RECT r;
 
@@ -2444,10 +2432,8 @@ BOOL WINAPI NtUserShowCaret( HWND hwnd )
 
     if (ret && hidden == 1)  /* hidden was 1 so it's now 0 */
     {
-        pt.x = r.left;
-        pt.y = r.top;
         display_caret( hwnd, &r );
-        set_ime_composition_window_pos( hwnd, &pt );
+        set_ime_composition_rect( hwnd, r );
         NtUserSetSystemTimer( hwnd, SYSTEM_TIMER_CARET, caret.timeout );
     }
     return ret;
