@@ -432,7 +432,7 @@ static const struct user_driver_funcs x11drv_funcs =
     .pToUnicodeEx = X11DRV_ToUnicodeEx,
     .pVkKeyScanEx = X11DRV_VkKeyScanEx,
     .pNotifyIMEStatus = X11DRV_NotifyIMEStatus,
-    .pSetIMECompositionWindowPos = X11DRV_SetIMECompositionWindowPos,
+    .pSetIMECompositionRect = X11DRV_SetIMECompositionRect,
     .pDestroyCursorIcon = X11DRV_DestroyCursorIcon,
     .pSetCursor = X11DRV_SetCursor,
     .pGetCursorPos = X11DRV_GetCursorPos,
